﻿using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Common;
using HeuristicLab.Core;

namespace HeuristicLab.Encodings.SymbolicExpressionTreeEncoding {
  [Item("LaTeX/PDF Formatter", "Formatter for symbolic expression trees for use with latex.")]
  public class SymbolicExpressionTreeLatexFormatter : NamedItem, ISymbolicExpressionTreeStringFormatter {
    private readonly static Dictionary<string, string> symbolNameMap = new Dictionary<string, string>
    {
      {"ProgramRootSymbol", "Prog"},
      {"StartSymbol","RPB"}
    };
    private readonly ReingoldTilfordLayoutEngine<ISymbolicExpressionTreeNode> layoutEngine;
    private readonly SymbolicExpressionTreeLayoutAdapter layoutAdapter;


    public SymbolicExpressionTreeLatexFormatter() {
      layoutEngine = new ReingoldTilfordLayoutEngine<ISymbolicExpressionTreeNode>();
    }

    protected SymbolicExpressionTreeLatexFormatter(SymbolicExpressionTreeLatexFormatter original, Cloner cloner)
      : base(original, cloner) {
      layoutEngine = new ReingoldTilfordLayoutEngine<ISymbolicExpressionTreeNode>();
      layoutAdapter = new SymbolicExpressionTreeLayoutAdapter();
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new SymbolicExpressionTreeLatexFormatter(this, cloner);
    }

    public string Format(ISymbolicExpressionTree symbolicExpressionTree) {
      var layoutNodes = layoutAdapter.Convert(symbolicExpressionTree).ToList();
      layoutEngine.Reset();
      layoutEngine.Root = layoutNodes[0];
      foreach (var ln in layoutNodes)
        layoutEngine.AddNode(ln.Content, ln);
      var nodeCoordinates = layoutEngine.GetNodeCoordinates();
      var sb = new StringBuilder();
      sb.AppendLine("\\begin{figure}");
      sb.AppendLine("\\begin{tikzpicture}");

      foreach (var node in symbolicExpressionTree.IterateNodesBreadth()) {
        var symbolName = node.Symbol.Name;
        var nodeName = symbolNameMap.ContainsKey(symbolName) ? symbolNameMap[symbolName] : symbolName;
        var coord = nodeCoordinates[node];

        sb.AppendLine(string.Format("\\draw node ({0},{1}) {{{2}}};", coord.X, coord.Y, nodeName));
      }

      sb.AppendLine("\\end{figure}");
      sb.AppendLine("\\end{tikzpicture}");

      return sb.ToString();
    }
  }
}
